/*
===============================================================================
=== Released under GPL (GNU Public License) ==== No Any Warranty ==============
===============================================================================
*/

#include "../include/common.h"
#include "../include/config.h"

void OzEDIT_SaveConfiguration() {
    FILE *f;
    f = fopen(OZEDITOR_CONFIGFILE,"w");
    if(f == NULL) {
	OzEDIT_ErrorDialog("Cannot create/write configuration file %s !",OZEDITOR_CONFIGFILE);
	return;
    }
    /* Ora procedi a salvare la configurazione in pseudo-XML */
    fprintf(f,"! OzEDITor Configuration -  http://ozeditor.zerozone.it \n");
    fprintf(f,"! v%02d.%02d.%02d \n",OZEDIT_VERSION,OZEDIT_SUBVERSION,OZEDIT_PATCH);
    /* Oggetti */
    fprintf(f,"! Configuration Section \n");    
    fprintf(f,"<Configuration>\n");

    if(OzEDIT_Config.autocheck_for_updates) { fprintf(f,"  <AutoCheckForUpdates \"YES\">\n"); } else { fprintf(f,"  <AutoCheckForUpdates \"NO\">\n"); }
    if(OzEDIT_Config.colorize_code) { fprintf(f,"  <ColorizeCode \"YES\">\n"); } else { fprintf(f,"  <ColorizeCode \"NO\">\n"); }
    fprintf(f,"  <FontTextBuffer \"%s\">\n",pango_font_description_to_string(OzEDIT_Config.font_textbuffer));	
    fprintf(f,"</Configuration>\n");    
    fprintf(f,"! End \n");    
    fclose(f);
}

void OzEDIT_LoadConfiguration() {
    struct _OzGtkXML *XMLRoot, *XMLTemp;
    XMLRoot = (struct _OzGtkXML *)OzGtkXML_ParseFile(OZEDITOR_CONFIGFILE);
    if(XMLRoot == NULL) { 
#ifdef DEBUG
        g_print("Configuration FILE  %s not found - Using DEFAULTS \n",OZEDITOR_CONFIGFILE);
#endif    
	OzEDIT_Config.autocheck_for_updates = TRUE;
	OzEDIT_Config.colorize_code = TRUE;
        OzEDIT_Config.font_textbuffer = pango_font_description_from_string("Sans 12");
    } else {
        XMLTemp = (struct _OzGtkXML *)OzGtkXML_GetElementFromName(XMLRoot,"Configuration");
	if(g_ascii_strcasecmp((gchar *)OzGtkXML_GetElementValue(XMLTemp,"AutoCheckForUpdates"),"NO") == 0) { 
	    OzEDIT_Config.autocheck_for_updates = FALSE;
	} else {
	    OzEDIT_Config.autocheck_for_updates = TRUE;
	}
	if(g_ascii_strcasecmp((gchar *)OzGtkXML_GetElementValue(XMLTemp,"ColorizeCode"),"NO") == 0) { 
	    OzEDIT_Config.colorize_code = FALSE;
	} else {
	    OzEDIT_Config.colorize_code = TRUE;
	}
        OzEDIT_Config.font_textbuffer = pango_font_description_from_string((gchar *)OzGtkXML_GetElementValue(XMLTemp,"FontTextBuffer"));    
    }
}


void OzEDIT_CBConfigFontButton(GtkButton *button,GtkWidget *textentry) {
    GtkWidget *config_dialog_font;
    
    config_dialog_font = gtk_font_selection_dialog_new("Choose font");
    gtk_dialog_run(GTK_DIALOG(config_dialog_font));	
    
    OzEDIT_Config.font_textbuffer = pango_font_description_from_string(gtk_font_selection_dialog_get_font_name(GTK_FONT_SELECTION_DIALOG(config_dialog_font)));

    gtk_entry_set_text(GTK_ENTRY(textentry),pango_font_description_to_string(OzEDIT_Config.font_textbuffer));
    
    gtk_widget_destroy(config_dialog_font);    

}

void OzEDIT_ConfigureDialog() {
#warning "ConfigurDialog() - CHECK FOR BUGS"
    GtkWidget *config_dialog,*config_table,*config_label;
    GtkWidget *config_check_autoupdate,*config_check_colorize,*config_fontdialog_textbuffer,*config_textentry_textbuffer;
    GtkWidget *config_button_font;
    	
    config_dialog = gtk_dialog_new_with_buttons("Configuration",NULL,GTK_DIALOG_DESTROY_WITH_PARENT,GTK_STOCK_OK,GTK_RESPONSE_ACCEPT,NULL);
    gtk_window_set_default_size(GTK_WINDOW(config_dialog),200,50);
	
    config_table = gtk_table_new(5,2,FALSE);
    gtk_container_add(GTK_CONTAINER(GTK_DIALOG(config_dialog)->vbox),config_table);

    config_check_autoupdate = gtk_check_button_new_with_label("Automatically check for updates");
    if(OzEDIT_Config.autocheck_for_updates) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config_check_autoupdate),TRUE);
    gtk_table_attach_defaults(GTK_TABLE(config_table),config_check_autoupdate,0,2,1,2); /* LEFT - RIGHT - TOP - BOTTOM */	
    gtk_widget_show(config_check_autoupdate);

    config_check_colorize = gtk_check_button_new_with_label("Colorize source code");
    if(OzEDIT_Config.colorize_code) gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(config_check_colorize),TRUE);
    gtk_table_attach_defaults(GTK_TABLE(config_table),config_check_colorize,0,2,2,3); /* LEFT - RIGHT - TOP - BOTTOM */	
    gtk_widget_show(config_check_colorize);

    config_textentry_textbuffer = gtk_entry_new();
    gtk_entry_set_text(GTK_ENTRY(config_textentry_textbuffer),pango_font_description_to_string(OzEDIT_Config.font_textbuffer));
    gtk_entry_set_editable(GTK_ENTRY(config_textentry_textbuffer),FALSE);
    gtk_table_attach_defaults(GTK_TABLE(config_table),config_textentry_textbuffer,0,2,3,4); /* LEFT - RIGHT - TOP - BOTTOM */
    gtk_widget_show(config_textentry_textbuffer);    
    
    config_button_font = gtk_button_new_with_label("Choose Font");
    gtk_table_attach_defaults(GTK_TABLE(config_table),config_button_font,0,1,4,5); /* LEFT - RIGHT - TOP - BOTTOM */
    g_signal_connect(GTK_OBJECT(config_button_font),"pressed",G_CALLBACK(OzEDIT_CBConfigFontButton),config_textentry_textbuffer);

    gtk_widget_show(config_button_font); 
    
    gtk_widget_show(config_table);
    
    gtk_dialog_run(GTK_DIALOG(config_dialog));	
    /* Controlla lo stato dei widget e modifica la configurazione */
    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(config_check_autoupdate))) { 
	OzEDIT_Config.autocheck_for_updates = TRUE; 
    } else {
	OzEDIT_Config.autocheck_for_updates = FALSE;
    }
    if(gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(config_check_colorize))) { 
	OzEDIT_Config.colorize_code = TRUE; 
    } else {
	OzEDIT_Config.colorize_code = FALSE;
    }
    OzEDIT_Config.font_textbuffer = pango_font_description_from_string(gtk_entry_get_text(GTK_ENTRY(config_textentry_textbuffer)));
    /* Chiudi la Finestra */ 
    gtk_widget_destroy(config_dialog);    
    OzEDIT_SaveConfiguration();
}
