/*
===============================================================================
=== Released under GPL (GNU Public License) ==== No Any Warranty ==============
===============================================================================
*/

#include "../include/common.h"
#include "../include/workspace.h"

struct _OzGtkXML *OzGtkXML_ParseFile(gchar *);

void ExtractElements(struct _OzGtkXML *XML) {
    struct _OzGtkXML *XMLObjects,*XMLBookMarks;
    struct _OzEDIT_Object *object;    
    struct _OzEDIT_Bookmark *bookmark;
    GtkTextIter iter;
    guint n,c;
    gint line,lineoff;
    for(n=0;n<g_list_length(XML->Elements);n++) {
	XMLObjects = g_list_nth_data(XML->Elements,n);
#ifdef DEBUG
	g_print("%d - %s(%s)\n",n,XMLObjects->name,XMLObjects->value);
#endif
	if(g_ascii_strncasecmp(XMLObjects->name,"Object",strlen(XMLObjects->name))==0) {
	    /* Carica l'oggetto */
    	    object = (struct _OzEDIT_Object *)OzEDIT_CreateNewObject(OzGtkXML_GetElementValue(XMLObjects,"FileName"));
	    for(c=0;c<g_list_length(XMLObjects->Elements);c++) {
	    	XMLBookMarks = g_list_nth_data(XMLObjects->Elements,c);
		if(g_ascii_strncasecmp(XMLBookMarks->name,"Bookmark",strlen(XMLBookMarks->name))==0) {
		    /* Carica il Bookmark */
		    bookmark = g_malloc0(sizeof(struct _OzEDIT_Bookmark));
		    g_strlcpy(bookmark->bk_name,(gchar *)OzGtkXML_GetElementValue(XMLBookMarks,"BKName"),64);
		    g_strlcpy(bookmark->name,(gchar *)OzGtkXML_GetElementValue(XMLBookMarks,"Name"),64);
		    /* Crea l'ITER */
		    line = atoi((gchar *)OzGtkXML_GetElementValue(XMLBookMarks,"Line"));
		    lineoff = atoi((gchar *)OzGtkXML_GetElementValue(XMLBookMarks,"LineOff"));
		    gtk_text_buffer_get_iter_at_line_offset(object->text_buffer,&iter,line,lineoff);
		    /* Aggiungere Mark al TextBuffer */
		    gtk_text_buffer_create_mark(object->text_buffer,bookmark->name,&iter,TRUE);
		    /* Aggiornare lista ogetti */
		    gtk_tree_store_append(GTK_TREE_STORE(topclass.treestore),&bookmark->treeiter,&object->treeiter);
		    gtk_tree_store_set(GTK_TREE_STORE(topclass.treestore),&bookmark->treeiter,0,bookmark->bk_name,-1);
#ifndef _NOGDKPIXBUF
		    /* Aggiungere il PIXBUF al TextBuffer */
		    gtk_text_iter_forward_to_line_end(&iter);
		    gtk_text_buffer_insert_pixbuf(object->text_buffer,&iter,topclass.pixbuf_bookmark);
#endif
		    /* Aggiungi alla lista... */
		    object->bookmarks = g_list_append(object->bookmarks,bookmark);
		    object->bk_count++;
		}
	    }	    
	}	
    }
}

/* === Open Workspace ================================== */
void OzEDIT_OpenWorkspace(gchar *wsfilename) {
    struct _OzGtkXML *XMLRoot;
    XMLRoot = OzGtkXML_ParseFile(wsfilename);
    if(XMLRoot == NULL) {
	OzEDIT_ErrorDialog("Cannot read file %s !",wsfilename);
	return;
    }
    ExtractElements(XMLRoot);
}

/* === Save Workspace ================================== */
void OzEDIT_SaveWorkspace(gchar *wsfilename) {
    struct _OzEDIT_Object *object;
    struct _OzEDIT_Bookmark *bookmark;
    GtkTextIter iter;
    FILE *f;
    guint i,ii;
    if(!g_str_has_suffix(wsfilename,".ows")) { /* l'estenzione OWS non e' stata messa, aggiungila */
	g_strlcat(wsfilename,".ows",255); /* SEGFAULT ? */
    }
    f = fopen(wsfilename,"w");
    if(f == NULL) {
	OzEDIT_ErrorDialog("Cannot create/write file %s !",wsfilename);
	return;
    }
    /* Ora procedi a salvare il WorkSpace in pseudo-XML */
    fprintf(f,"! OzEDITor WorkSpace http://ozeditor.zerozone.it \n");
    fprintf(f,"! v%02d.%02d.%02d \n",OZEDIT_VERSION,OZEDIT_SUBVERSION,OZEDIT_PATCH);
    /* Oggetti */
    fprintf(f,"! Objects Section \n");    
    for(i=0;i<g_list_length(topclass.objects);i++) {
	object = g_list_nth_data(topclass.objects,i);
	fprintf(f,"<Object>\n");
	fprintf(f,"  <Name \"%s\">\n",object->name);
	fprintf(f,"  <FileName \"%s\">\n",object->filename);	
        for(ii=0;ii<g_list_length(object->bookmarks);ii++) {
	    bookmark = g_list_nth_data(object->bookmarks,ii);
	    gtk_text_buffer_get_iter_at_mark(object->text_buffer,&iter,gtk_text_buffer_get_mark(object->text_buffer,bookmark->name));
    	    fprintf(f,"  <Bookmark>\n");
	    fprintf(f,"     <Name \"%s\">\n",bookmark->name);	
	    fprintf(f,"     <BKName \"%s\">\n",bookmark->bk_name);
	    fprintf(f,"     <Line \"%d\">\n",gtk_text_iter_get_line(&iter));
	    fprintf(f,"     <LineOff \"%d\">\n",gtk_text_iter_get_line_offset(&iter));
    	    fprintf(f,"  </Bookmark>\n");		    
	}
	fprintf(f,"</Object>\n");
    }
    fprintf(f,"! End \n");    
    fclose(f);
}

/* === Close Workspace ================================== */
void OzEDIT_CloseWorkspace(gchar *wsfilename) {

}

